/**  SearchAgent - versione riveduta, adattata e ampliata dell'agente "Shell".
 *     Agente in grado di realizza un tour fra i place contenuti in una tabella
 *		 per tornare in fine al place di origine! Durante il tour ricerca 
 *		 informazioni codificate in XML secondo lo standard di metadati Dublin Core
 *     @author     Emiliano Zappacosta
 *     @version    1.00
 *     @language   jdk 1.2.2
 */


import SOMA.gui.FinAgentMessageListener;
import SOMA.Environment;
import SOMA.agent.*;
import SOMA.naming.*;
import SOMA.utility.*;
import SOMA.mobilePlace.*;

import java.io.File;
import java.io.ObjectOutputStream;
import java.util.Arrays;
import java.util.Vector;
import java.util.Hashtable;
import java.net.InetAddress;


public class SearchAgent extends Agent implements FinAgentMessageListener {

  // VARIABILI NON SERIALIZZATE:
      /**
       *  L'oggetto che realizza il motore di ricerca sulle info 
       *	XML-Dublin Core. Esso viene creato localmente su ogni place
       *	dove apre il file dei dati.
       */
	protected transient dcXMLSearcher searcher;

	protected transient Vector arg;
	
	
	

  // VARIABILI SERIALIZZATE:
      /**
       *  Il "chronometer", per calcolare il tempo impiegato per la migrazione
       *  tra un Place e l'altro.
       */
  protected Chronometer chronometer = new Chronometer();

      /**
       *  Elenco di tutti i Place visitati.
       *  Questa struttura cresce di un elemento a ogni migrazione!
       */
//  protected Vector placeVisitati;
      /**
       *  Elenco della storia passata: contiene uno dopo l'altro tutti i Place
       *  su cui l'agente  passato.
       *  Questa struttura cresce di un elemento a ogni migrazione!
       */
  //protected Vector placeStorici;

      /**
       *  Elenco di tutti i Place da visitare durante il Tour.
       *  Questa struttura decresce di un elemento a ogni migrazione!
       */
  protected Vector placeTour;

      /**
       *  Indice che identifica il Place successivo da visitare nel Tour.
       *  Zero indica che non ci sono altri place da visitare: si torna 
       * 	a casa!
       */
  protected int placeNext = -1;

      /**
       *  Qui ci metto il PlaceID a cui dovrei trovarmi dopo un'azione di "GOTO".
       *  Nello "startMethod" controllo che il Place attuale sia effettivamente questo qui.
       *  Se non lo , significa che l'azione di GO NON HA AVUTO PIENO SUCCESSO: dal Place 
       *  precedente si  migrati a un Place che non  quello a cui si voleva andare, poich
       *  la migrazione si  INTERROTTA AL PLACE DI DEFAULT DEL DOMINIO IN CUI SI ERA.
       */
  protected PlaceID placeSuCuiMigrare;
	protected PlaceID home;
	protected AgentID parent; // placeid dell'agente Scarpet che mi ha creato

  Hashtable searchContent;
 	Vector result;

 
 
 
/* 	=====================================================
		*  	metodi della classe SearcheAgent								*
		===================================================== */

/**
 *  Metodo che permette di definire lo stato iniziale dell'agente alla creazione.
 *  Definisce e inizializza alcuni oggetti usati.
 */
  public void putArgument(Object a){
			arg = (Vector) a;
  }

  
/**
 *  Metodo di partenza dell'agente all'atto della creazione.
 *  Definisce e inizializza tutti gli oggetti usati e alla fine chiama "startMethod".
 */
  public void run () {
    agentSystem.getOut().println (getID() + " - run");

    placeSuCuiMigrare = null;
 		home = getID().getHome();
		String place = (String)arg.get(0);
		parent = (AgentID)arg.get(1);
		searchContent = (Hashtable)arg.get(2);
    agentSystem.getOut().println (getID() + " - "+searchContent);

  	if(place == "RicercaSequenziale") {
  			placeTour = new Vector( Arrays.asList(agentSystem.getPlaces()) );
				placeTour.remove(home);
				placeNext = placeTour.size()-1;
  	}
  	else {	//l'agente visita un solo nodo
  			try{
  				PlaceID p = new PlaceID(place);
  				placeTour = new Vector( );
  				placeTour.add(p);
  				placeNext = 0;
  			}
		  	catch (Exception e) {
				  agentSystem.getOut().println (" Errore ... " + e);}
  	}

    if (mailbox != null)    // ATTENTO! SE  "NON TRACEABLE" NON C'E' LA MAILBOX!
      mailbox.mailListener = new Mailbox.MailListener() {
          public void run(){
/*            if (winOutFrame != null)
              winOutFrame.out.println (Lingua.frase("THEAG_TXT_GOT_MAIL"));*/
          }
      };
      
  
//********* inizializzazione dati x la ricerca ********* 	

  result = new Vector();
  
  
   chronometer.Start();
   startMethod();
  } //run

    
      

/**
 *  Questo  il metodo di avvio dopo "Idle" e "go" (migrazione).
 */
  public void startMethod () {
    agentSystem.getOut().println (getID() + " - startMethod ");

    PlaceID placeAtt = agentSystem.getPlaceID();    // PLACE CORRENTE (SU CUI MI TROVO)

    // CONTROLLO SE L'AGENTE E' RIUSCITO A MIGRARE FINO A DOVE VOLEVA!
    if (placeSuCuiMigrare != null) 
      if (! placeAtt.equals(placeSuCuiMigrare)) {  // NON CE L'HO FATTA!
        agentSystem.getOut().println (" Problemi di migrazione"+" '"+
                                    placeSuCuiMigrare.toString()+"'");
      }
        
  /*  // AGGIORNO GLI ELENCHI DEI PLACE VISITATI E LO STORICO
    // Avevo gi visitato questo placeID? ( nella lista dei visitati?)
    int posPreced = placeVisitati.indexOf (placeAtt);
    if (posPreced < 0)  // -1 Non c': l'aggiungo, se no la sostituisco.
      placeVisitati.addElement (placeAtt);
    //else
      //placeVisitati.setElementAt (placeAtt, posPreced);
    placeStorici.addElement (placeAtt);       // Nella placeStorici c' tutta la sequenza.
*/
	
	try{
  	if (! placeAtt.equals(home)) {	// Solo al lancio  falsa la condizione
  		// aggiorno la tabella del percorso
  		placeTour.remove(placeNext);
  		placeNext--; 
 			
 			// Creazione e inizializzazione del searcher
    	Environment env = agentSystem.getEnvironment();
    	String dirIniAgente = env.agentManager.agentClassManager.getClassPathDirectory();
    	String fileDati = dirIniAgente + File.separator + 
    										placeAtt.place + "data.xml";   
	 		searcher = new dcXMLSearcher( fileDati );
			
			// Fase di ricerca dei dati  
  		agentSystem.getOut().print (getID()+" - searching  : "+fileDati+"  ");
 			Vector R = searcher.search(searchContent);
 			agentSystem.getOut().println (R.size());

			result.add(R);
  	} //  endif (! placeAtt.equals(home))
  	
  	
  	//Fase di spostamento dell'agente
  	try {
  		if (placeNext >= 0)  // ci sono altre tappe da fare
  				// Parto alla volta della prossima tappa....
         	eseguiAzioneGoto ((PlaceID) placeTour.elementAt(placeNext), "startMethod");	
  			
  		else {	// le tappe sono finite (<0);Torno a casa; il Tour  finito ;-(
  	 			PlaceID placeCasa = getID().getHome();
		  		placeNext--; 
       		eseguiAzioneGoto (placeCasa, "homeMethod");
    			}
  	}
  	catch (Exception e) {
			  agentSystem.getOut().println (" Errore ...di migarzione " + e);
    }


  }
	catch( Exception e){                     	//IOException 
			  agentSystem.getOut().println (" Errore ... " + e);
  			}
	
  } //startMethod

      

/**
 *  Questo  il metodo di avvio al ritono dalla ricerca.
 */
  public void homeMethod () {
		Vector R = new Vector();
   	Vector S;		
   	Float crono;
   	
 		agentSystem.getOut().println (getID() + " - homeMethod ");
		
		//----Aggiornamento cronometro globale
		chronometer.Stop();
		Float C = new Float((chronometer.StopTime - chronometer.StartTime) / 1000);
   	agentSystem.getOut().println ("Search Time - " + C);
		crono = (Float)agentSystem.sharedObjects.get("Time");
		if(crono.floatValue()<C.floatValue())  {
			crono=C;}
		agentSystem.sharedObjects.put("Time",crono);

		//----Aggiornamento risultato ricerca
		for (int i = result.size() -1; i>=0; i--){
			S = (Vector)result.get(i);
			for (int j = S.size() -1; j>=0; j--)
				R.add( S.get(j) );
		}
		S = (Vector)agentSystem.sharedObjects.get("ScarpetAgent");
		S.addAll(R);
		agentSystem.sharedObjects.put("ScarpetAgent", S);

   	Message mes = new Message("SearchAgent_result", getID(), parent);
   	try{
   		eseguiAzioneSend( mes );
   	}
   	catch (Exception e){
			agentSystem.getOut().println (e);}

		agentSystem.getOut().println (R.size()+"  "+R);
	} //homeMethod
  

/**
 *  Metodo invocato per effettuare la migrazione dell'agente.
 */ 
  public void eseguiAzioneGoto (PlaceID placeToGoTo, String method) throws Exception {
    // NB: l'eccezione viene "presa" dalla finestra FinGoto1000, che provveder a stamparla in una finestra

    PlaceID placeAtt = agentSystem.getPlaceID();

		if (! placeToGoTo.equals(placeAtt)) {

        PlaceID mobilePlaceID = MobilePlaceID.parsePlaceID (placeToGoTo.toString());
        placeSuCuiMigrare = mobilePlaceID;   // Mi ricorder il place su cui volevo migrare.

        go (mobilePlaceID, method);
    }
		// Sedevo migrare sullo stesso place non faccio nulla
  } //eseguiAzioneGoto


 

/**
 *  Metodo invocato dalla FinAgentMessage dopo che l'utente preme il bottone "send".
 *  Tale finestra  aperta dal "ListenerMenu".
 */ 
  public void eseguiAzioneSend (Message message) throws Exception {
    agentSystem.sendMessage (message);
  } //eseguiAzioneSend





} //SearchAgent