//Title:       dcXML-Searcher
//Version:     1.0
//Copyright:   Copyright (c) 2001
//Author:      E.Zappacosta
//Company:    
//Description: dcXMLSearcher definisce un analizzatore sintattico per
//						 lfile di metadati Dublin Core codificati in XML; legge 
//						 i token della frase ed effettua subito la ricerca


/* ==========================================================================
	dcXMLSearcher   definisce un analizzatore sintattico in definitiva che
									compie azioni di confronto
  ============================================================================== */
import java.io.FileReader;
import java.io.Reader;
import java.io.BufferedReader;
import java.util.Hashtable;
import java.util.Vector;


class dcXMLSearcher{
	
	dcXMLLexer L;					// Lexer
	String curToken;			//termine correntemente letto dal file XML
	
//struttura che contiene tutti i termini da ricercare raggruppati per tag
// (key) per esempio <type>; gli oggetti associati alle key sono dei vettori
// contenenti tutti i termini associati al tag.
	Hashtable content;	
	
	String contentTag;		//contenuto di un tag DC di un record del file XML
	Vector v;							//vettore di appoggio x termini da ricercare dei vari Tag DC

//numero di punti accumulati con la presenza di termini corretti nei vari 
// tag di un record DC sotto esame
	int point;	

//soglia da superare x i punti di un record xke il record sia rilevante
// ai fini della ricerca che si sta effettuando
	int soglia;
	
//vettore di appoggio x contenere i valori TITOLO-AUTORE-DESCRIZIONE-IDENTIFICATORE
// di un record DC potenzialmente rilevante 
	Vector risPot;

//vettore dei risultati; contiene i vari risPot risultati rilevanti
	Vector result;
		
/* ************** Riconoscitore della grammatica ************* 
			
			DCS	::= <dublin-core-simple> RL </dublin-core-simple>
			RL	::= <record-list> LIST </record-list>
			LIST::= DCR | DCR LIST 
			DCR	::= <dc-record> 1 o + tag dublin core </dc-record>    
			
   ***********************************************************	*/			
	
	public dcXMLSearcher(String filename) throws Exception{
			Reader r = new BufferedReader( new FileReader(filename) );
	 		L = new dcXMLLexer( r );
	 		contentTag = new String();
	 		v = new Vector();
	 		result = new Vector();
	}// costruttore Parser
		
	
	public Vector search(Hashtable c) throws Exception{
		content = c;
	//	System.out.println(c);

		// calcolo della soglia di rilevanza
		soglia =0;
		if ((v = (Vector)content.get("<type>")) != null)
				soglia += 10000*v.size();
		if ((v = (Vector)content.get("<creator>")) != null)
				soglia += 1000*v.size();
		if ((v = (Vector)content.get("<title>")) != null)
				soglia += 100*v.size();
		if ((v = (Vector)content.get("<language>")) != null)
				soglia += 10*v.size();
		if ((v = (Vector)content.get("<subject>")) != null)
				soglia += v.size();	//vale 1

		// inizio parsing del file XML
		curToken = L.next();
		DCS();
		return result;
	}// search()
		


// ======== Dublin Core Simple ===============================
	protected void DCS()throws Exception{
		if( curToken.equals("<dublin-core-simple>") ) {
			 			curToken = L.next();	//	consumo il token corrente
						RL();
						if( curToken.equals("</dublin-core-simple>") )
								curToken = L.next();	//	consumo il token corrente
						else 
							throw new Exception("not <dublin-core-simple> file");
 						}
		else
			throw new Exception("not <dublin-core-simple> file");
	}// DCS()
			
		

// ======== Record List ======================================
	protected void RL()throws Exception{
		if( curToken.equals("<record-list>") ) {
				curToken = L.next();	//	consumo il token corrente
				LIST();
				if( curToken.equals("</record-list>") )
						curToken = L.next();	//	consumo il token corrente
				else 
						throw new Exception("not <record-list> structure");
 				}
		else
			throw new Exception("not <record-list> structure");
	}// RL()
	


// ======== LIST ===============================================
	protected void LIST()throws Exception{
		for(; curToken.equals("<dc-record>"); ) 
			DCR();
	}// LIST()
	
	

// ======== Dublin Core Record =================================
	protected void DCR() throws Exception{
		if( curToken.equals("<dc-record>") ) {
				curToken = L.next();	//	consumo il token corrente
				point = 0;
		 		risPot = new Vector();
				
				for(; ; ) 
					if( curToken.equals("<type>") ) 						point += TY();
					else if( curToken.equals("<title>") ) 			point += TI();
					else if( curToken.equals("<format>") ) 			point += FO();
					else if( curToken.equals("<description>") ) point += DE();
					else if( curToken.equals("<subject>") ) 		point += SU();
					else if( curToken.equals("<creator>") ) 		point += CR();
					else if( curToken.equals("<publisher>") ) 	point += PU();
					else if( curToken.equals("<contributor>") ) point += CON();
					else if( curToken.equals("<date>") ) 				point += DA();
					else if( curToken.equals("<coverage>") ) 		point += COV();
					else if( curToken.equals("<identifier>") ) 	point += ID();
					else if( curToken.equals("<source>") ) 			point += SO();
					else if( curToken.equals("<relation>") ) 		point += RE();
					else if( curToken.equals("<language>") ) 		point += LA();
					else if( curToken.equals("<rights>") ) 			point += RI();
					else 																				break;
					
			/*	System.out.println( point );
				System.out.println( risPot );*/

				if( curToken.equals("</dc-record>") ) {
					curToken = L.next();	//	consumo il token corrente
					if (point >= soglia) result.add(risPot);
				}
				else 
					throw new Exception("abnormal <dc-record> structure");
 				}
		else
			throw new Exception("not <dc-record> structure");
	}//   DCR()
	


// ======== TYpe ==========================================
	protected int TY()throws Exception{
			int t=0;
			contentTag = ""; 

			curToken = L.next();	//	consumo il token corrente

			for (;!( curToken.equals("</type>") || curToken.equals("EOF")); curToken = L.next())
						contentTag +=curToken+" ";
			if( !curToken.equals("</type>") )
						throw new Exception("abnormal <type> structure");

			curToken = L.next();	//	consumo il token corrente

			if ((v = (Vector)content.get("<type>")) != null)
				for(int i=v.size()-1; i>=0; i--)
						if (contentTag.indexOf( (String)v.get(i)) != -1)  t+=10000;
			return t;							
		}// TY()


// ======== TItle ========================================
	protected int TI()throws Exception{
			int t=0;
			contentTag = ""; 

			curToken = L.next();	//	consumo il token corrente

			for (; !(curToken.equals("</title>") || curToken.equals("EOF"));curToken = L.next())
									contentTag +=curToken+" ";
			if( !curToken.equals("</title>") )
						throw new Exception("abnormal <title> structure");

			curToken = L.next();	//	consumo il token corrente
			
			if ((v = (Vector)content.get("<title>")) != null)
				for(int i=v.size()-1; i>=0; i--)
						if (contentTag.indexOf( (String)v.get(i)) != -1)   t+=100;
			risPot.add(new String(contentTag));
			return t;							
	}// TI()


// ======== FOrmat ========================================
	protected int FO()throws Exception{
			int t=0;
			contentTag = ""; 
			
			curToken = L.next();	//	consumo il token corrente

			for (; !(curToken.equals("</format>") || curToken.equals("EOF"));curToken = L.next())
									contentTag +=curToken+" ";
			if( !curToken.equals("</format>") )
						throw new Exception("abnormal <format> structure");

			curToken = L.next();	//	consumo il token corrente
			
			if ((v = (Vector)content.get("<format>")) != null)
				for(int i=v.size()-1; i>=0; i--)
						if (contentTag.indexOf( (String)v.get(i)) != -1) t++;
			return t;							
	}// FO()
		
		
// ======== DEscription ===================================
	protected int DE()throws Exception{
			contentTag = ""; 

			curToken = L.next();	//	consumo il token corrente

			for (; !(curToken.equals("</description>") || curToken.equals("EOF"));curToken = L.next())
									contentTag +=curToken+" ";
			if( !curToken.equals("</description>") )
						throw new Exception("abnormal <description> structure");

			curToken = L.next();	//	consumo il token corrente
			risPot.add(new String(contentTag));
			return 0;
	}// DE()
		
		
// ======== SUbject =======================================
	protected int SU()throws Exception{
			int t=0;
			contentTag = ""; 

			curToken = L.next();	//	consumo il token corrente

			for (; !(curToken.equals("</subject>") || curToken.equals("EOF"));curToken = L.next())
									contentTag +=curToken+" ";
			if( !curToken.equals("</subject>") )
						throw new Exception("abnormal <subject> structure");

			curToken = L.next();	//	consumo il token corrente
			if ((v = (Vector)content.get("<subject>")) != null)
				for(int i=v.size()-1; i>=0; i--)
						if (contentTag.indexOf( (String)v.get(i)) != -1) t++;
			return t;						
	}// SU()
		
		
// ======== CReator =======================================
	protected int CR()throws Exception{
			int t=0;
			contentTag = ""; 
		
			curToken = L.next();	//	consumo il token corrente

			for (; !(curToken.equals("</creator>") || curToken.equals("EOF"));curToken = L.next())
									contentTag +=curToken+" ";
			if( !curToken.equals("</creator>") )
						throw new Exception("abnormal <creator> structure");

			curToken = L.next();	//	consumo il token corrente
			if ((v = (Vector)content.get("<creator>")) != null)
				for(int i=v.size()-1; i>=0; i--)
						if (contentTag.indexOf( (String)v.get(i)) != -1)   t+=1000;
			risPot.add(new String(contentTag));
			return t;							

	}// CR()
		
		
// ======== PUblisher =====================================
	protected int PU()throws Exception{
			curToken = L.next();	//	consumo il token corrente

			for (; !(curToken.equals("</publisher>") || curToken.equals("EOF"));
						 curToken = L.next());
			if( !curToken.equals("</publisher>") )
						throw new Exception("abnormal <publisher> structure");

			curToken = L.next();	//	consumo il token corrente
			return 0;				
	}// PU()
		
		
// ======== CONtributor ===================================
	protected int CON()throws Exception{
			curToken = L.next();	//	consumo il token corrente

			for (; !(curToken.equals("</contributor>") || curToken.equals("EOF"));
						 curToken = L.next());
			if( !curToken.equals("</contributor>") )
						throw new Exception("abnormal <contributor> structure");

			curToken = L.next();	//	consumo il token corrente
			return 0;				
	}// CON()


// ======== DAte ========================================
	protected int DA()throws Exception{
			int t=0;
			contentTag = ""; 

			curToken = L.next();	//	consumo il token corrente

			for (; !(curToken.equals("</date>") || curToken.equals("EOF"));curToken = L.next())
									contentTag +=curToken+" ";
			if( !curToken.equals("</date>") )
						throw new Exception("abnormal <date> structure");

			curToken = L.next();	//	consumo il token corrente
							if ((v = (Vector)content.get("<date>")) != null)
				for(int i=v.size()-1; i>=0; i--)
						if (contentTag.indexOf( (String)v.get(i)) != -1) t++;
			return t;							
	}// DA()
		
		
// ======== COVerage ====================================
	protected int COV()throws Exception{
			int t=0;
			contentTag = ""; 
		
			curToken = L.next();	//	consumo il token corrente

			for (; !(curToken.equals("</coverage>") || curToken.equals("EOF"));curToken = L.next())
									contentTag +=curToken+" ";
			if( !curToken.equals("</coverage>") )
						throw new Exception("abnormal <coverage> structure");

			curToken = L.next();	//	consumo il token corrente
							if ((v = (Vector)content.get("<coverage>")) != null)
				for(int i=v.size()-1; i>=0; i--)
						if (contentTag.indexOf( (String)v.get(i)) != -1) t++;
			return t;							
	}// COV()
		
		
// ======== IDentifer ===================================
	protected int ID()throws Exception{
			contentTag = ""; 

			curToken = L.next();	//	consumo il token corrente

			for (; !(curToken.equals("</identifier>") || curToken.equals("EOF"));curToken = L.next())
									contentTag +=curToken+" ";
			if( !curToken.equals("</identifier>") )
						throw new Exception("abnormal <identifier> structure");

			curToken = L.next();	//	consumo il token corrente
			risPot.add(new String(contentTag));
			return 0;				
	}// ID()
		
		
// ======== SOurce ======================================
	protected int SO()throws Exception{
			curToken = L.next();	//	consumo il token corrente

			for (; !(curToken.equals("</source>") || curToken.equals("EOF"));
						 curToken = L.next());
			if( !curToken.equals("</source>") )
						throw new Exception("abnormal <source> structure");

			curToken = L.next();	//	consumo il token corrente
			return 0;				
	}// SO()
		
		
// ======== RElation ========================================
	protected int RE()throws Exception{
			curToken = L.next();	//	consumo il token corrente

			for (; !(curToken.equals("</relation>") || curToken.equals("EOF"));
						 curToken = L.next());
			if( !curToken.equals("</relation>") )
						throw new Exception("abnormal <relation> structure");

			curToken = L.next();	//	consumo il token corrente
			return 0;				
	}// RE()
		
		
// ======== LAnguage =======================================
	protected int LA()throws Exception{
			int t=0;
			contentTag = ""; 
		
			curToken = L.next();	//	consumo il token corrente

			for (; !(curToken.equals("</language>") || curToken.equals("EOF"));curToken = L.next())
									contentTag +=curToken+" ";
			if( !curToken.equals("</language>") )
						throw new Exception("abnormal <language> structure");

			curToken = L.next();	//	consumo il token corrente
							if ((v = (Vector)content.get("<language>")) != null)
				for(int i=v.size()-1; i>=0; i--)
						if (contentTag.indexOf( (String)v.get(i)) != -1)   t+=10;
			return t;							
	}// LA()
		
		
// ======== RIghts ========================================
	protected int RI()throws Exception{
			curToken = L.next();	//	consumo il token corrente

			for (; !(curToken.equals("</rights>") || curToken.equals("EOF"));
						 curToken = L.next());
			if( !curToken.equals("</rights>") )
						throw new Exception("abnormal <rights> structure");

			curToken = L.next();	//	consumo il token corrente
			return 0;				
	}// RI()


}// class dcXMLSearcher